/*********************************************************************************************************************
 *
 *		Azoteq IQS525-TP43-HID-Ev-Kit
 *
 *              IQS525_handler.c
 *              Read and analyse data from the IQS device
 *
********************************************************************************************************************/

#include "includes.h"      


#include "includes.h"

// Variables for filtering
// beta-states of filter defined as an enumerated type
typedef enum BETA{ SLOW = 0, FAST = 1} BETA;
// current beta-state
BETA current_beta;
#define SLOW_BETA 0.3
#define FAST_BETA 0.9
coordinate avg;

// absolute value function
#define abs(n)   ( ((n) >= 0) ? (n) : -1*(n) )

// data buffer for communication with the touchpad
unsigned char data_buffer[30];

// variables for touchpad state
signed int X;
signed int Y;
unsigned int absolute_X;
unsigned int absolute_Y;
unsigned int absolute_prevX;
unsigned int absolute_prevY;

// event handling
unsigned char previous_event;

// circular buffer for relative movement
extern relative_movement_buffer relative_movement_buf;

// boolean to determine whether the pointer may be moved or not
extern BOOL MOUSE_MOVEMENT_ALLOWED;

// keyboard
BOOL LEFT_KEY;
BOOL RIGHT_KEY;
BOOL UP_KEY;
BOOL DOWN_KEY;
BOOL ENTER_KEY;
BOOL RELEASED_KEY; // boolean to help generate the key released event

/*
 * IQS 525 variables
 */
unsigned char NoOfFingers;
unsigned char XYInfoByte;
unsigned char prevXYInfoByte;
//struct array to store finger data
IQS5XX_FINGER fingers[5];

// variables to identify the cursor id
unsigned char current_cycle_main_id;
unsigned char previous_cycle_main_id;
unsigned char cursor_id;            // if the main id stays the same for 2 consecutive cycles == choose as cursor id
BOOL MAIN_ID_CHOSEN;

// variable to count startup cycles -- need to get relative coordinates
unsigned char startup_cycle_counter;

// snap variables
unsigned int SnapStatus[TOTALTXS_VAL<<1];
BOOL PREVIOUS_CYCLE_SNAP;
BOOL CURRENT_CYCLE_SNAP;
BOOL SNAP_POST_BUFFER_ACTIVE;
unsigned char snap_counter;
#define SNAP_POST_BUFFER_SIZE 35
unsigned char toprow;
unsigned char middlerow;
unsigned char bottomrow;
unsigned char rightchar;
unsigned char downchar;

//click variables
int click_cycle_counter;
unsigned char ID_air_track;
int cycles_on_trackpad;
BOOL COUNT_CYCLES_ON_TRACKPAD;
int lift_x;
int lift_y;
int land_x;
int land_y;
unsigned char left_click;



/*********************************************************************************************************************
 *                      --- void () ---
 *          This function identifies the connected touchpad device and performs setup
 *          An identification byte is set to enable to microcontroller adapt the filtering
********************************************************************************************************************/

void identify_and_setup_IQS(void)
{
        // setup IQS525
        setup_IQS525();

        // variables used to identify cursor ID
        current_cycle_main_id = 0;
        previous_cycle_main_id = 0;
        cursor_id = 0;
        MAIN_ID_CHOSEN = FALSE;

        // counter used to obtain relative coordinates
        startup_cycle_counter = 0;

        // variables used to obtain relative coordinates
        X = 0;
        Y = 0;
        absolute_X = 0;
        absolute_Y = 0;
        absolute_prevX = 0;
        absolute_prevY = 0;

        // variables used for click functionality
        click_cycle_counter = 0;
        ID_air_track = 'a'; // start in air
        cycles_on_trackpad =0;
        COUNT_CYCLES_ON_TRACKPAD = FALSE;
        lift_x = 0;
        lift_y = 0;
        land_x = 0;
        land_y = 0;

        // initialise the relative movement buffer (for smooth mouse movement)
        initialise_circular_buffer (&relative_movement_buf, 'a');

        // initialise the touchpad with a slow beta
        current_beta = SLOW;

        // initialise click byte
        left_click = 0x00;

        // allow mouse movement
        MOUSE_MOVEMENT_ALLOWED = TRUE;

        // variables for keyboard
        RELEASED_KEY = TRUE;
        LEFT_KEY = FALSE;
        RIGHT_KEY = FALSE;
        UP_KEY = FALSE;
        DOWN_KEY = FALSE;
        ENTER_KEY = FALSE;
        PREVIOUS_CYCLE_SNAP = FALSE;
        CURRENT_CYCLE_SNAP = FALSE;
        toprow = 0x00;
        middlerow = 0x00;
        bottomrow = 0x00;
        SNAP_POST_BUFFER_ACTIVE = FALSE;
        snap_counter = 0;

}
/*********************************************************************************************************************
 *                      --- void filter_movement () ---
 *          This function filters the new movement placing a weight on the history
********************************************************************************************************************/
void filter_movement (void)
{
    // calculate average coordinate in buffer
   calculate_average_coordinate (&relative_movement_buf, &avg);

    // filter using the dynamic beta
    if (current_beta == SLOW)
    {
       X = X*SLOW_BETA+ (1-SLOW_BETA)*avg.X;
       Y = Y*SLOW_BETA+ (1-SLOW_BETA)*avg.Y;
    }
    else
    {
       X = X*FAST_BETA+ (1-FAST_BETA)*avg.X;
       Y = Y*FAST_BETA+ (1-FAST_BETA)*avg.Y;
    }
}

/*********************************************************************************************************************
 *                      --- void determine_beta () ---
 *          This function decides whether to use a fast or a slow beta
********************************************************************************************************************/
void determine_beta (void)
{
    int count = 0;

    // If all the touch has been moving in the same direction for the last 'BUFFER_SIZE' samples, swith to a fast beta

    // obtain array with movement history - most recent movement at index 0
    coordinate movement_history[MOVEMENT_BUFFER_SIZE];
    create_array_copy_circular_buffer( &relative_movement_buf, &movement_history[0]);

    //startup handler
   if (relative_movement_buf.noe < MOVEMENT_BUFFER_SIZE)
    {
        current_beta = SLOW;
        return;
    }

    current_beta = FAST;

    // handle X
    while (count < MOVEMENT_BUFFER_SIZE)
    {
        if (movement_history[0].X > 0)
        {
            if (movement_history[count].X < 0)
            {
                current_beta = SLOW;
            }
        }
        else
        {
          if (movement_history[count].X > 0)
          {
                current_beta = SLOW;
          }
        }
        count = count + 1;
    }
    count = 0;
    //handle Y
    while (count < MOVEMENT_BUFFER_SIZE)
    {
        if (movement_history[0].Y > 0)
        {
            if (movement_history[count].Y < 0)
            {
                current_beta = SLOW;
            }
        }
        else
        {
          if (movement_history[count].Y > 0)
          {
                current_beta = SLOW;
          }
        }
        count = count + 1;
    }

    if ((movement_history[0].Y == 0) || (movement_history[0].X == 0))
            current_beta = SLOW;

}

/****************************************************************************************************************************
 * this function handles events - snaps for keyboard keys
 ****************************************************************************************************************************/
void handle_events(void)
{

    if ((XYInfoByte & 0b00001000) == 0b00001000)
    {
        CURRENT_CYCLE_SNAP = TRUE;
        MOUSE_MOVEMENT_ALLOWED = FALSE;
        // debounce and then copy snap status
            toprow = SnapStatus[0]&0xFF;
            middlerow = SnapStatus[2]&0xFF ;
            bottomrow = SnapStatus[3]&0xFF;

            SNAP_POST_BUFFER_ACTIVE = TRUE;
            X = 0;
            Y = 0;
    }
    else
    {
       // LATD = 0xFF;
        CURRENT_CYCLE_SNAP = FALSE;
    }

    if ((CURRENT_CYCLE_SNAP == FALSE) && (PREVIOUS_CYCLE_SNAP == TRUE))
    {

        MOUSE_MOVEMENT_ALLOWED = FALSE;
        //up event
        if (toprow == 4)
        {
            UP_KEY = TRUE;
        }
        //left
        if (middlerow == 16)
        {
            LEFT_KEY = TRUE;
        }
        // enter
        if (middlerow == 4)
        {
            ENTER_KEY = TRUE;
        }
        // right
        if (middlerow == 1)
        {
            RIGHT_KEY = TRUE;
        }
        // down
        if (bottomrow == 4)
        {
            DOWN_KEY = TRUE;
        }

    }
    PREVIOUS_CYCLE_SNAP = CURRENT_CYCLE_SNAP;
}

/*********************************************************************************************************************
 *                      --- void serviceTouchpad(void) ---
 *          main touchpad service routine
********************************************************************************************************************/
void serviceTouchpad(void)
{       unsigned char counter = 0;
        // coordinates used for calculations
        coordinate temp_coordinate;

        // snap postbuffer
        if (SNAP_POST_BUFFER_ACTIVE == TRUE)
        {
            snap_counter = snap_counter + 1;
            if (snap_counter == SNAP_POST_BUFFER_SIZE)
            {
                snap_counter = 0;
                MOUSE_MOVEMENT_ALLOWED = TRUE;
                clear_all_circular_buffer(&relative_movement_buf);

                MAIN_ID_CHOSEN = FALSE;



                SNAP_POST_BUFFER_ACTIVE = FALSE;
            }
            else
                return;
        }

        // update variables monitoring the touchpad
        readTouchpadState ();

        // event handler goes here
        handle_events();



        if(NoOfFingers > 0)
        {
            // identify the cursor ID from all the fingers (assume one)
            current_cycle_main_id = fingers[0].ID;
            if((current_cycle_main_id == previous_cycle_main_id) && (MAIN_ID_CHOSEN == FALSE))
            {
                cursor_id = current_cycle_main_id;
                MAIN_ID_CHOSEN = TRUE;
            }

            // We have identified the main cursor
            if (MAIN_ID_CHOSEN == TRUE)
            {
                // If we have identified a hover ID, restart identification procedure
                if( current_cycle_main_id >5)
                {
                    MAIN_ID_CHOSEN = FALSE;
                    set_main_ID_state ('a');
                    startup_cycle_counter = 0;
                    X =0;
                    Y = 0;
                    return;
                }
                else
                {
                    set_main_ID_state('t');
                }


                // obtain absolute X and Y coordinates from cursor ID
                getXY (cursor_id);

                // rotate axis
                absolute_X = -1*absolute_X;

                // this is not a valid condition, restart
                if (absolute_X == -1 && absolute_Y == -1)
                {
                    MAIN_ID_CHOSEN = FALSE;
                    startup_cycle_counter = 0;
                }


                // this will execute the second cycle after the cursor id has been chosen == relative coordinates
                if (startup_cycle_counter == 2)
                {
                    // calculate relative coordinates
                    X = absolute_X - absolute_prevX;
                    Y = absolute_Y - absolute_prevY;

                   // filter movement
                    filter_movement();

                    // store in movement history buffer (relative coordinates)
                    temp_coordinate.X = absolute_X - absolute_prevX;
                    temp_coordinate.Y = absolute_Y - absolute_prevY;
                    temp_coordinate.active = 'y';
                    enqueue_in_circular_buffer( &relative_movement_buf, &temp_coordinate);

                    // determine the new filter beta
                    determine_beta();
                }
                else
                {
                    startup_cycle_counter = startup_cycle_counter + 1;
                }
                absolute_prevX = absolute_X;
                absolute_prevY = absolute_Y;
            }
            previous_cycle_main_id = current_cycle_main_id;

        }
        else
        {
            set_main_ID_state('a');
            // this is a prox event or no event at all, reset variables
            MAIN_ID_CHOSEN = FALSE;
            startup_cycle_counter = 0;
            // no movement of pointer
            X = 0;
            Y  = 0;
        }


}
/*********************************************************************************************************************
 *                      --- void getXY(unsigned char ID) ---
 *          Extract the XY coordinates associated with an ID from the finger arraye
********************************************************************************************************************/
void getXY(unsigned char ID)
{
    int count = 0;
    while (count < 5)
    {
        if (fingers[count].ID == ID)
        {
            absolute_X =  fingers[count].Xpos;
            absolute_Y = fingers[count].Ypos;
            return;
        }
        count++;
    }
}
/*********************************************************************************************************************
 *                      --- void set_main_ID_state(unsigned char idstring) ---
 *                      set variable to indicate if finger is on the trackpad or if finger is in the air
 *                      from this we generate the click condition
 *                      clicks are generated with finger taps: if finger touches the trackpad, and then leaves it in almost the same area in a short time period
********************************************************************************************************************/
void set_main_ID_state(unsigned char idstring)
{
    if (ID_air_track == 'a' && idstring =='t')
    {
        // finger just touched
        COUNT_CYCLES_ON_TRACKPAD = TRUE;
        getXY(current_cycle_main_id);
        land_x = absolute_X;
        land_y = absolute_Y;
    }
    else if (idstring == 'a' && ID_air_track == 't')
    {
        //finger just lifted
        getXY(current_cycle_main_id);
        lift_x = absolute_X;
        lift_y = absolute_Y;
        if (cycles_on_trackpad < 120 && COUNT_CYCLES_ON_TRACKPAD == TRUE)
        {
            if (abs(lift_x - land_x) < 100 && abs(lift_y - land_y) < 100)
            {
                //click event
                X = land_x;
                Y = land_y;
                left_click = 0x01;
            }
        }
        cycles_on_trackpad = 0;
        COUNT_CYCLES_ON_TRACKPAD = FALSE;
    }
    if (COUNT_CYCLES_ON_TRACKPAD == TRUE)
    {
            cycles_on_trackpad++;
     }
    ID_air_track = idstring;
}

/*********************************************************************************************************************
 *                      --- void readTouchpadState (void) ---
 *          Read variables from touchpad
********************************************************************************************************************/
void readTouchpadState (void)
{
	unsigned char data_buffer[37], i, j;

	// Read the XY data, but only read as many XY co-ordinates as shown in the XYInfoByte:
	i = 0;

        // read the first byte
	CommsIQS_start();
        CommsIQS_send((IQS5xx_ADDR << 1) + 0x00);
	CommsIQS_send(XY_DATA);
	CommsIQS_repeat_start();
	CommsIQS_send((IQS5xx_ADDR << 1) + 0x01);
        data_buffer[i++] = CommsIQS_read_ack();	// Read first byte (XYInfo)

	//process XYInfoBYte data:
	if ((XYInfoByte & SHOW_RESET) != 0)						// check for an unexpected reset.
	{
		data_buffer[i] = CommsIQS_read_nack();		// end the read in progress
		CommsIQS_stop();

                delay_ms(100);
		// handle the reset
                identify_and_setup_IQS();
                LATD = 0x00;
                return;
	}

	prevXYInfoByte = XYInfoByte;
	XYInfoByte = data_buffer[0];
	NoOfFingers = XYInfoByte & 0x0F;



	if (NoOfFingers > 0)
	{
		for (j = 0; j < ((NoOfFingers*7)-1); j++)
		{
			data_buffer[i++] = CommsIQS_read_ack();
		}
		data_buffer[i++] = CommsIQS_read_nack();	// last read must have a NACK
	}
	else
		data_buffer[i] = CommsIQS_read_nack();		// MUST read another because must..
															// ..know about NACK beforehand.

        // sort the received XY data in structure
	for (i = 0; i < NoOfFingers; i++)
	{
		fingers[i].ID = data_buffer[(i*7)+1];

		fingers[i].Xpos = (int)(data_buffer[(i*7)+2])<<8;
		fingers[i].Xpos |= data_buffer[(i*7)+3];

		fingers[i].Ypos = (int)(data_buffer[(i*7)+4])<<8;
		fingers[i].Ypos |= data_buffer[(i*7)+5];

		fingers[i].TouchStrength = (int)(data_buffer[(i*7)+6])<<8;
		fingers[i].TouchStrength |= data_buffer[(i*7)+7];
	}

	if ((XYInfoByte & SNAP_OUTPUT) != 0)					// if there are active snap outputs
	{
		CommsIQS_repeat_start();
		CommsIQS_Read(IQS5xx_ADDR, SNAP_STATUS, &data_buffer[0], (TOTALTXS_VAL<<1)); // 2 bytes per Tx for snap status

		// sort the received SNAP data
		for (i = 0; i < TOTALTXS_VAL; i++)
			SnapStatus[i] = (((unsigned int)data_buffer[i<<1])<<8) + ((unsigned int)data_buffer[(i<<1) + 1]); // add the upper and lower bytes to get the full word
	}
	else
	{
		for (i = 0; i < TOTALTXS_VAL; i++)
			SnapStatus[i] = 0x0000; 						// no snaps, so set registers all to zero
	}
	CommsIQS_stop();
}


/*********************************************************************************************************************
 *                      --- void setup_IQS525 (void) ---
 *          Load IQS525 settings
********************************************************************************************************************/
void setup_IQS525 (void)
{
        unsigned char data_buffer[30];

        delay_ms(100);

        // ACKNOWLEDGE THE RESET INDICATION FLAG
	data_buffer[0] = ACK_RESET;						// Set ACK_RESET flag, which clears SHOW_RESET in XYInfoByte
	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, CONTROL_SETTINGS, &data_buffer[0], 1);
	CommsIQS_stop();								// Now the bit is clear, and if ever SHOW_RESET is set..


        // ..an unexpected reset can be handled
        // -----------------------------------------------------
        // FIRSTLY SETUP THE CHANNELS USED:

	// ChannelSetup Data:
	data_buffer[0] = TOTALRXS_VAL;					// TotalRx
	data_buffer[1] = TOTALTXS_VAL;					// TotalTx
	data_buffer[2] = TRACKPADRXS_VAL;				// TrackPadRx
	data_buffer[3] = TRACKPADTXS_VAL;				// TrackPadTx
	data_buffer[4] = PMSETUP0_VAL;					// PMSetup register
	data_buffer[5] = TXHIGH_VAL;
	data_buffer[6] = TXLOW_VAL;						// Which Tx's are used for the ProxMode channel (projected only)

	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, CHANNEL_SETUP, &data_buffer[0], 7);
	CommsIQS_stop();

       // -----------------------------------------------------
        // CONFIGURE THE THRESHOLDS:

	// Threshold settings data
	data_buffer[0] = PROXTHRESHOLD_VAL;				// Prox Threshold
	data_buffer[1] = TOUCHMULTIPLIER_VAL;			// Touch Multiplier
	data_buffer[2] = TOUCHSHIFTER_VAL;				// Touch Shifter
	data_buffer[3] = PMPROXTHRESHOLD_VAL;			// PM Prox Threshold
	data_buffer[4] = (unsigned char)(SNAPTHRESHOLD_VAL>>8);	// Snap threshold
	data_buffer[5] = (unsigned char)SNAPTHRESHOLD_VAL;		// Snap threshold
	data_buffer[6] = PROXTHRESHOLD2_VAL;		// Non-trackpad channels prox threshold
	data_buffer[7] = TOUCHMULTIPLIER2_VAL;			// Non-trackpad channels Touch Multiplier
	data_buffer[8] = TOUCHSHIFTER2_VAL;				// Non-trackpad channels Touch Shifter

	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, THRESHOLD_SETTINGS, &data_buffer[0], 9); // send thresholds
	CommsIQS_stop();

        // -----------------------------------------------------
        // SETUP THE ATI PARAMETERS FOR NORMAL MODE, AND EXECUTE AUTO-ATI ALGORITHM;

	// ATI Settings Data:
	data_buffer[0] = (unsigned char)(ATITARGET_VAL>>8);
	data_buffer[1] = (unsigned char)ATITARGET_VAL;		// ATI Target
	data_buffer[2] = ATIC_VAL;							// ATI C
	data_buffer[3] = (unsigned char)(ATITARGET2_VAL>>8);
	data_buffer[4] = (unsigned char)ATITARGET2_VAL;		// Non-trackpad channels ATI Target
	data_buffer[5] = ATIC2_VAL;							// Non-trackpad channels ATI C

	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, ATI_SETTINGS, &data_buffer[0], 6); // Write the ATI Parameters
	data_buffer[0] = AUTO_ATI;							 // Set Auto_ATI, with Mode set to Normal Mode.
	CommsIQS_repeat_start();
	CommsIQS_Write(IQS5xx_ADDR, CONTROL_SETTINGS, &data_buffer[0], 1);
	CommsIQS_stop();									 // Now IQS5xx will permform the Auto-ATI algorithm

        // -----------------------------------------------------
        // SETUP THE FILTER PARAMETERS;

	// Filter Settings Data:
	data_buffer[0] = FILTERSETTINGS0_VAL;				// Numerous filter settings
	data_buffer[1] = TOUCHDAMPING_VAL;					// XY touch point filtering parameter
	data_buffer[2] = HOVERDAMPING_VAL;					// XY hover point filtering parameter
	data_buffer[3] = PMCOUNTDAMPING_VAL;      			// ProxMode count value filter parameter (full-speed)
	data_buffer[4] = LPPMCOUNTDAMPING_VAL;				// ProxMode count value filter parameter (Low Power)
	data_buffer[5] = NMCOUNTDAMPING_VAL;				// Normal Mode count value filter parameter

	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, FILTER_SETTINGS, &data_buffer[0], 6);
	CommsIQS_stop();

        // -----------------------------------------------------
        // SETUP THE TIMING PARAMETERS;

	// Timing Settings Data:
	data_buffer[0] = RESEEDTIME_VAL;					// LTA reseed timer
	data_buffer[1] = COMMSTIMEOUT_VAL;					// Inactive i2c timeout value
	data_buffer[2] = MODETIME_VAL;						// Mode timer value (Switching between modes time)
	data_buffer[3] = LPTIME_VAL;						// Low power time added in low-power state
	data_buffer[4] = SLEEPTIME_VAL;						// Sleep time permanently added

	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, TIMING_SETTINGS, &data_buffer[0], 5);
	CommsIQS_stop();

        // -----------------------------------------------------
        // SETUP THE HARDWARE PARAMETERS;

	// Hardware Config Settings Data:
	data_buffer[0] = PROXSETTINGS0_VAL;
	data_buffer[1] = PROXSETTINGS1_VAL;
	data_buffer[2] = PROXSETTINGS2_VAL;
	data_buffer[3] = PROXSETTINGS3_VAL;

	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, HW_CONFIG_SETTINGS, &data_buffer[0], 4);
	CommsIQS_stop();

        // -----------------------------------------------------
        // SETUP THE ACTIVE CHANNELS

	// Active Channels Data:
	data_buffer[0] = (unsigned char)(ACTIVECHANNELS0_VAL>>8);
	data_buffer[1] = (unsigned char)ACTIVECHANNELS0_VAL;
	data_buffer[2] = (unsigned char)(ACTIVECHANNELS1_VAL>>8);
	data_buffer[3] = (unsigned char)ACTIVECHANNELS1_VAL;
	data_buffer[4] = (unsigned char)(ACTIVECHANNELS2_VAL>>8);
	data_buffer[5] = (unsigned char)ACTIVECHANNELS2_VAL;
	data_buffer[6] = (unsigned char)(ACTIVECHANNELS3_VAL>>8);
	data_buffer[7] = (unsigned char)ACTIVECHANNELS3_VAL;
	data_buffer[8] = (unsigned char)(ACTIVECHANNELS4_VAL>>8);
	data_buffer[9] = (unsigned char)ACTIVECHANNELS4_VAL;
	data_buffer[10] = (unsigned char)(ACTIVECHANNELS5_VAL>>8);
	data_buffer[11] = (unsigned char)ACTIVECHANNELS5_VAL;
	data_buffer[12] = (unsigned char)(ACTIVECHANNELS6_VAL>>8);
	data_buffer[13] = (unsigned char)ACTIVECHANNELS6_VAL;
	data_buffer[14] = (unsigned char)(ACTIVECHANNELS7_VAL>>8);
	data_buffer[15] = (unsigned char)ACTIVECHANNELS7_VAL;
	data_buffer[16] = (unsigned char)(ACTIVECHANNELS8_VAL>>8);
	data_buffer[17] = (unsigned char)ACTIVECHANNELS8_VAL;
	data_buffer[18] = (unsigned char)(ACTIVECHANNELS9_VAL>>8);
	data_buffer[19] = (unsigned char)ACTIVECHANNELS9_VAL;
	data_buffer[20] = (unsigned char)(ACTIVECHANNELS10_VAL>>8);
	data_buffer[21] = (unsigned char)ACTIVECHANNELS10_VAL;
	data_buffer[22] = (unsigned char)(ACTIVECHANNELS11_VAL>>8);
	data_buffer[23] = (unsigned char)ACTIVECHANNELS11_VAL;
	data_buffer[24] = (unsigned char)(ACTIVECHANNELS12_VAL>>8);
	data_buffer[25] = (unsigned char)ACTIVECHANNELS12_VAL;
	data_buffer[26] = (unsigned char)(ACTIVECHANNELS13_VAL>>8);
	data_buffer[27] = (unsigned char)ACTIVECHANNELS13_VAL;
	data_buffer[28] = (unsigned char)(ACTIVECHANNELS14_VAL>>8);
	data_buffer[29] = (unsigned char)ACTIVECHANNELS14_VAL;

	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, ACTIVE_CHANNELS, &data_buffer[0], 30);
	CommsIQS_stop();

        // -----------------------------------------------------
        // SETUP THE DEBOUNCE SETTINGS;

	// DebounceSettings Data:
	data_buffer[0] = PROXDB_VAL;						// Prox Debounce values
	data_buffer[1] = TOUCHSNAPDB_VAL;					// Touch and Snap debounce values

	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, DB_SETTINGS, &data_buffer[0], 2);
	CommsIQS_stop();

        // -----------------------------------------------------
        // SETUP THE ATI PARAMETERS FOR PROX MODE, AND EXECUTE AUTO-ATI ALGORITHM

	// First change the mode to ProxMode:
	data_buffer[0] = MODE_SELECT;						// Set Mode bit to ProxMode
	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, CONTROL_SETTINGS, &data_buffer[0], 1);
	CommsIQS_stop();									// now next cycle will be in ProxMode

	// ProxMode ATI Settings Data:
	data_buffer[0] = (unsigned char)(PMATITARGET_VAL>>8);
	data_buffer[1] = (unsigned char)PMATITARGET_VAL;	// PM ATI Target
	data_buffer[2] = PMATIC_VAL;						// PM ATI C

	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, PM_ATI_SETTINGS, &data_buffer[0], 3); // Write the ATI Parameters
	data_buffer[0] = MODE_SELECT | AUTO_ATI;				// Keep ProxMode selected, and enable auto-ati
	CommsIQS_repeat_start();
	CommsIQS_Write(IQS5xx_ADDR, CONTROL_SETTINGS, &data_buffer[0], 1);
	CommsIQS_stop();										// go and perform PM Auto-ATI Routine

        // -----------------------------------------------------
        // NOW FINALLY SETUP THE LOW-POWER, SLEEP, SYSTEM MODE AND EVENT SETTINGS

	// Control Settings Data:
	data_buffer[0] = CONTROLSETTINGS0_VAL;
	data_buffer[1] = CONTROLSETTINGS1_VAL;
	CommsIQS_start();
	CommsIQS_Write(IQS5xx_ADDR, CONTROL_SETTINGS, &data_buffer[0], 2);
	CommsIQS_stop();

        // ---- Setup Complete -------

}

