
/*********************************************************************************************************************
 *
 *		Azoteq IQS525-TP43-HID-Ev-Kit
 *
 *              circular_buffer.c
 *              Used during the filter procedure to obtain average relative movement vectors
 *
********************************************************************************************************************/

#include "includes.h"

/* Define buffer for relative movement */
relative_movement_buffer relative_movement_buf;

/***************************************************************************************************************************/
/* Functions operating on coordinate struct*/
void clear_coordinate( coordinate *c)
{
	c->X = 0;
	c->Y = 0;
        c->active = 'n';
}

int distance_between_coordinates(coordinate *c1, coordinate *c2)
{
    if ((c1->active == 'y') && (c2->active == 'y'))
      return ((c2->X - c1->X)*(c2->X - c1->X) + (c2->Y - c1->Y)*(c2->Y - c1->Y));
    else
      return 0;
}

/*Functions operating on circular buffer struct */

/****************************************************************************************************************************
 *
 * Included types to ensure that code can be reused at a later stage
 *
 * type = 'a' for movement buffer
 ****************************************************************************************************************************/
void initialise_circular_buffer(relative_movement_buffer *cb, char type)
{
	int i = 0;

	/* initial pointer positions*/
	cb->front = 0;
	cb->back =  0;

	/* initial element count */
	cb->noe = 0;

        /* store length vector */
        if (type == 'a')
            cb->length = MOVEMENT_BUFFER_SIZE;

	/* clear 'placeholders' in buffer */
	while (i < cb->length)
	{
		clear_coordinate(&cb->buffer[i]);
		i++;
	}
       
}

/****************************************************************************************************************************
 *  Enqueue at the back of the buffer
 ****************************************************************************************************************************/
void enqueue_in_circular_buffer(relative_movement_buffer *cb, coordinate *cor)
{
	/* place elements at the back of the buffer, handle overflow*/
	cb->buffer[(cb->back)%cb->length] = *cor;
	cb->back = (cb->back + 1)%cb->length;

	/* count to maximum number of elements*/
	if (cb->noe != cb->length)
	{
		cb->noe++;
	}
	else
	{
		cb->front  = (cb->front + 1)%cb->length;  // update position of the oldest element
	}
}

/****************************************************************************************************************************
 *  Return handle to most recent element -- do not delete
 ****************************************************************************************************************************/
coordinate most_recent_element_circular_buffer (relative_movement_buffer *cb)
{
	if (cb->back == 0)
		return cb->buffer[cb->noe - 1];
	else
		return cb->buffer[cb->back - 1];
}
/****************************************************************************************************************************
 *  Return handle to oldest element -- do not delete
 ****************************************************************************************************************************/
coordinate oldest_element_circular_buffer(relative_movement_buffer *cb)
{
	return cb->buffer[cb->front];
}

/****************************************************************************************************************************
 *  Clear all the elements inside the buffer
 ****************************************************************************************************************************/
void clear_all_circular_buffer(relative_movement_buffer *cb)
{
    if (cb->length == MOVEMENT_BUFFER_SIZE)
	initialise_circular_buffer(cb, 'a');
}

/****************************************************************************************************************************
  Fill the array with the elements inside the circular buffer, index 0 corresponds to the most recent element
   Note - array must point to a sufficient memory location
****************************************************************************************************************************/
void create_array_copy_circular_buffer(relative_movement_buffer *cb, coordinate *array)
{
	int i = 0;
	int cb_counter = cb->back-1;

	// copy 'noe' elements
	while (i < cb->noe)
	{
		//handle the overflow case
		if (cb_counter == -1)
			cb_counter = cb->noe -1;

		array[i++] = cb->buffer[cb_counter--];
	}
}

/****************************************************************************************************************************
 *  Calculate average coordinate
 ****************************************************************************************************************************/
void calculate_average_coordinate ( relative_movement_buffer *cb, coordinate *avg)
{
	signed int avg_x = 0;
	signed int avg_y = 0;
	int i = 0;

	// calculate average coordinate
	while (i < cb->length)
	{
		avg_x = avg_x  + cb->buffer[i].X;
		avg_y = avg_y + cb->buffer[i].Y;

		i++;
	}
	avg_x = avg_x / cb->length;
	avg_y = avg_y / cb->length;

	// asign average coordinate
	avg->X = avg_x;
	avg->Y = avg_y;
}

/****************************************************************************/


