/*********************************************************************************************************************
 *
 *		Azoteq IQS525-TP43-HID-Ev-Kit
 *
 *              circular_buffer.h - This file contains structs to store the relative movement history of the pointer.
 *                                - Functions that operated on the datastructures are also included.
 *                                - The datastructures and the methods are used to remove noise from the pointer movement
 *
********************************************************************************************************************/

#ifndef CIRCULAR_BUFFER_H
#define	CIRCULAR_BUFFER_H

/*********************************************************************************************************************
*   Structs to store data
********************************************************************************************************************/

/* Coordinate structure: store the relative movement coordinate (X,Y) that was read from the IQS device during one read cycle */
typedef struct coordinate
{
	int X;
	int Y;
        char active;                                                            // Indicate if coordinate is used in static structure.  'y' for yes 'n' for no.
} coordinate;

/* Circular buffers and functions operating on it */
#define MOVEMENT_BUFFER_SIZE 5
typedef struct circular_buffer
{
	int front;                                                              // Index of oldest element
	int back;                                                               // Position where next data element will be placed
	int noe;                                                                // Number of active elements
        int length;                                                             // Length of the data array (MOVEMENT_BUFFER_SIZE)
        coordinate buffer[MOVEMENT_BUFFER_SIZE];                                // Data
} relative_movement_buffer;

/*********************************************************************************************************************
*   Prototypes of functions operating on circular buffer
********************************************************************************************************************/
void clear_coordinate( coordinate *c);                                          // Clear the value of coordinate.
int distance_between_coordinates(coordinate *c1, coordinate *c2);               // Return a distance metric between points.
void initialise_circular_buffer(relative_movement_buffer *cb, char type);       // Initialise the circular buffer
void enqueue_in_circular_buffer(relative_movement_buffer *cb, coordinate *cor); // Enqueue coordinate at the back of the buffer.
coordinate most_recent_element_circular_buffer (relative_movement_buffer *cb);  // Return handle to the most recent element in the buffer.
coordinate oldest_element_circular_buffer(relative_movement_buffer *cb);        // Return handle to the oldest element in the buffer
void clear_all_circular_buffer(relative_movement_buffer *cb);                   // Clear all coordinates inside the buffer.
/* Fill the array with the elements inside the circular buffer, index 0 corresponds to the most recent element
   Note - array must point to a sufficient memory location
*/
void create_array_copy_circular_buffer(relative_movement_buffer *cb, coordinate *array);
/* Calculate average coordinate*/
void calculate_average_coordinate ( relative_movement_buffer *cb, coordinate *avg);
/****************************************************************************/


#endif

