/*********************************************************************************************************************
 *
 *		Azoteq IQS525-TP43-HID-Ev-Kit
 *
 *              i2c_low_level.c
 *
********************************************************************************************************************/


#include "includes.h"

/****************************************************************************************************************************
 *  void Comms_init(void)
 *  Initialises the I2C module on the PIC18F4550
*************************************************************************************************************************** */
void Comms_init(void)						//Initiates i2c module on PIC18F4550
{
	SSPADD = 0x1C;							// settings for I2C frequency - 416kHz
	SSPSTAT |= 0x80;						// slew rate control for high speed (400kHz)

	TRISB = TRISB | 0x03;					//set TRISB<0> Make I2c SDA and SCL inputs

	PIR1bits.SSPIF = 0;						// reset the initial interrupt vector
	SSPCON1 = 0x28;							//enables i2c, set to master

}


/****************************************************************************************************************************
 *  void CommsIQS_send(unsigned char send_data)
 * 	Description:	Transmits the data byte given as parameter via the I2C of the PIC18F4550
 *				Note that the I2C communication channel must already be active before calling this function,
 *				as no start bits are included in this function
 *
 *	Parameters:		send_data - data byte to be transmitted via the I2C
 ****************************************************************************************************************************/
void CommsIQS_send(unsigned char send_data)
{

     

	SSPBUF = send_data;						// write transmit byte to buffer

	while (PIR1bits.SSPIF == 0)				// wait for transmit complete flag
	{}
	PIR1bits.SSPIF = 0;						//clear flag

	while (SSPCON2bits.ACKSTAT == 1)
	{}


}


/****************************************************************************************************************************
 *  unsigned char CommsIQS_read_ack(void)
 * 	Description:	Enables the Master Receive Mode of the I2C module on the PIC18F4550. The data received is
 *				returned and an (ACK) acknowledge is sent to the IQS to indicate that more byte read's will
 *				still take place.
 *
 *	Return:		The data received via the I2C
 ****************************************************************************************************************************/
unsigned char CommsIQS_read_ack(void)
{
	unsigned char temp;

	SSPCON2bits.RCEN = 1;					// enable master receiver mode
	while (PIR1bits.SSPIF == 0)				// wait for byte received flag
	{}
	PIR1bits.SSPIF = 0;						//clear flag

	while (SSPSTATbits.BF == 0)				// wait for buffer full flag (receive complete)
	{}
	temp = SSPBUF;							// store received byte

	SSPCON2bits.ACKDT = 0;					// enable ACK
	SSPCON2bits.ACKEN = 1;					// execute ACK sequence

	while (PIR1bits.SSPIF == 0)				// Wait for ACK transmission complete
	{}
	PIR1bits.SSPIF = 0;						//clear flag

	while (SSPCON2bits.ACKEN == 1) 			//verify acknowledge sequence is complete
	{}

	return temp;
}
/****************************************************************************************************************************
 *  unsigned char CommsIQS_read_nack(void)
 *  Description:	Enables the Master Receive Mode of the I2C module on the PIC18F4550. The data received is
 *				returned and a NACK acknowledge is sent to the IQS to indicate that this was the final read
 *				of the current continuous read block.
 *
 *	Return:		The data received via the I2C
 *
 ****************************************************************************************************************************/
unsigned char CommsIQS_read_nack(void)
{
	unsigned char temp;

	SSPCON2bits.RCEN = 1;					// enable master receiver mode
	while (PIR1bits.SSPIF == 0)             // wait for byte received flag
	{}
	PIR1bits.SSPIF = 0;			            //clear flag

	while (SSPSTATbits.BF == 0)             // wait for buffer full flag (receive complete
	{}
	temp = SSPBUF;                          // store received byte

	SSPCON2bits.ACKDT = 1;                  // enable NACK
	SSPCON2bits.ACKEN = 1;                  // execute NACK sequence

	while (PIR1bits.SSPIF == 0)             // Wait for NACK transmission complete
	{}
	PIR1bits.SSPIF = 0;		                //clear flag

	while (SSPCON2bits.ACKEN == 1) 			//verify acknowledge sequence is complete
	{}

	return temp;
}

/****************************************************************************************************************************
 *      void CommsIQS_start(void)void CommsIQS_start(void)
 *      A i2c START condition is generated
 ****************************************************************************************************************************/
void CommsIQS_start(void)
{
	unsigned int a = 0;

	while(PORTAbits.RA1 == 0)		// wait for the IQS to change from not ready to ready
	{}

	SSPCON2bits.SEN = 1;			//start condition

	while (PIR1bits.SSPIF == 0)		//wait for start condition to be generated
	{}
	PIR1bits.SSPIF = 0;				//clear flag

	while (SSPCON2bits.SEN == 1)	// verify start is complete
	{}

}



/****************************************************************************************************************************
 *          void CommsIQS_repeat_start(void)
 *          Description:	An i2c START condition is generated.
 *
 ****************************************************************************************************************************/
void CommsIQS_repeat_start(void)
{

	SSPCON2bits.RSEN = 1;			//start condition

	while (PIR1bits.SSPIF == 0)	//wait for start to be generated
	{	}

	PIR1bits.SSPIF = 0;				//clear flag

	while (SSPCON2bits.RSEN == 1) //verify that restart is complete
	{	}


	//success

}

/****************************************************************************************************************************
 *      void CommsIQS_stop(void)
 *      Description:	An i2c STOP condition is generated.
 ****************************************************************************************************************************/
void CommsIQS_stop(void)
{
	SSPCON2bits.PEN = 1;			//stop condition

	while (PIR1bits.SSPIF == 0)		//wait for stop condition to be generated
	{}
	PIR1bits.SSPIF = 0;				//clear flag

	while (SSPCON2bits.PEN == 1)	// verify stop is complete
	{}

	while(PORTAbits.RA1 == 1)		//wait for the IQS to change from ready to not ready
	{}
	while(PORTAbits.RA1 == 0)		// wait for the IQS to change from not ready to ready
	{}

}