/*********************************************************************************************************************
 *
 *		Azoteq IQS525-TP43-HID-Ev-Kit
 *
 *              my_usb_handler.c - call functions to read and analyse data from the IQS device.  Constructs appropriate
 *                               - HID reports and transmit them to the PC
 *
********************************************************************************************************************/

/*********************************************************************************************************************
*   Include section
********************************************************************************************************************/
#include "includes.h"                                                           // main include file

/*********************************************************************************************************************
*   Global variables section
********************************************************************************************************************/

/* HID keyboard and mouse variables */

USB_HANDLE USBOutHandle1 = 0;                                                   // USB handle for mouse and keyboard data received form the PC
USB_HANDLE USBInHandle1 = 0;                                                    // USB handle for mouse and keyboard data send to PC.
extern signed int  X;                                                           // Relative x movement
extern signed int  Y;                                                           // Relative y movement
extern unsigned char  left_click;                                               // Click byte -- contains button information

extern BOOL LEFT_KEY;                                                           // Boolean to indicate that the left arrow was pressed
extern BOOL RIGHT_KEY;                                                          // Boolean to indicate that the left arrow was pressed
extern BOOL UP_KEY;                                                             // Boolean to indicate that the up arrow was pressed
extern BOOL DOWN_KEY;                                                           // Boolean to indicate that the down arrow was pressed
extern BOOL ENTER_KEY;                                                          // Boolean to indicate that the enter key was pressed
extern BOOL RELEASED_KEY;                                                       // Boolean to indicate that the released key command has been send
BOOL MOUSE_MOVEMENT_ALLOWED;                                                    // Boolean to determine when pointer movement is allowed
extern BOOL SNAP_POST_BUFFER_ACTIVE;                                            // Boolean to indicate that the snap post buffer is active

/* Composite HID device variables */

// USB handles
USB_HANDLE USBOutHandle2 = 0;                                                   // USB handle for composite HID device data received form the PC
USB_HANDLE USBInHandle2 = 0;                                                    // USB handle for composite HID device data send to the PC

// Variables streamed to composite HID device for debugging
extern int X;
extern int Y;
extern int XYInfoByte;
extern IQS5XX_FINGER fingers[5];
extern int startup_cycle_counter;
extern BOOL MAIN_ID_CHOSEN;
extern coordinate avg;
extern unsigned char ID_air_track;
extern int cycles_on_trackpad;
extern relative_movement_buffer relative_movement_buf;
extern unsigned char middlerow;                                                 // Byte to indicate the status of the buttons in the middle row of the TP43S


/*********************************************************************************************************************
*   Code section
********************************************************************************************************************/

/****************************************************************************************************************************
 *  void processIO (void):
 *  Main execution loop
 *  The execution of this function is interleaved with the general usb stack activities.
 *  This function reads the touchpad state and prepares the hid_buffer.
 *  The contents of the HID buffer is then send via the USB to the PC.
 ****************************************************************************************************************************/
void processIO (void)
{
    int i = 0;

    // Ensure device is configured correctly
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1))
    {
        return;
    }

     // ------------------------------------------------------------------------
     // IQS DEVICE

     // Communicate with the touchpad
     serviceTouchpad ();
    
     // ------------------------------------------------------------------------
     // POINTING DEVICE

    /*
     *   Construct pointing device hid report
     */

     // Construct a mouse hid report
     Emulate_Mouse();

     // Construct keyboard hid report when mouse movement is not allowed
     if (MOUSE_MOVEMENT_ALLOWED == FALSE)
           Emulate_Keyboard();
   
     /*
      *   Transmit pointing device hid report
      */

     // Send a HID report to the PC using the USB
     if (!HIDTxHandleBusy(USBInHandle1))
     {
           // Pointing device
           USBInHandle1 = HIDTxPacket(HID_EP1, (BYTE*)hid_report_in1, 9);
     }

     // ------------------------------------------------------------------------
    // Debugging, not used
     if(!HIDRxHandleBusy(USBOutHandle2))
     {

        // send command back to the GUI
	if(!HIDTxHandleBusy(USBInHandle2))
	{
            USBInHandle2 = HIDTxPacket(HID_EP2,(BYTE*)&hid_report_in2[0],64);
	}

        //Re-arm the OUT endpoint so that it can handle the next packet from the GUI
        USBOutHandle2 = HIDRxPacket(HID_EP2,(BYTE*)&hid_report_out2[0],64);
    }
}




/****************************************************************************************************************************
 *  void Emulate_Keyboard(void):
 *  This function constructs the keyboard HID report
 *
 *   Bit structure of byte 1:
 *          // 0 LEFT CTRL
 *         // 1 LEFT SHIFT
 *         // 2 LEFT ALT
 *         // 3 LEFT GUI
 *         // 4 RIGHT CTRL
 *         // 5 RIGHT SHIFT
 *         // 6 RIGHT ALT
 *         // 7 RIGHT GUI
 ****************************************************************************************************************************/
void Emulate_Keyboard(void)
{
    if ((LEFT_KEY == TRUE)&& (RELEASED_KEY == TRUE))
    {
        LEFT_KEY = FALSE;
        RELEASED_KEY = FALSE;


        // Load the HID buffer with the key
        hid_report_in1[0] = 0x02;
        //keyboard hid id
        hid_report_in1[1] = 0x00;
        hid_report_in1[2] = 0x00;
        hid_report_in1[3] = 0x50;
        hid_report_in1[4] = 0x00;
        hid_report_in1[5] = 0x00;
        hid_report_in1[6] = 0x00;
        hid_report_in1[7] = 0x00;
        hid_report_in1[8] = 0x00;

    }
    else if ((RIGHT_KEY == TRUE)&& (RELEASED_KEY == TRUE))
    {
        RIGHT_KEY = FALSE;
        RELEASED_KEY = FALSE;

        // Load the HID buffer with the key
        hid_report_in1[0] = 0x02; //keyboard hid id
        hid_report_in1[1] = 0x00;
        hid_report_in1[2] = 0x00;
        hid_report_in1[3] = 0x4F;
        hid_report_in1[4] = 0x00;
        hid_report_in1[5] = 0x00;
        hid_report_in1[6] = 0x00;
        hid_report_in1[7] = 0x00;
        hid_report_in1[8] = 0x00;

    }
    else if ((UP_KEY == TRUE)&& (RELEASED_KEY == TRUE))
    {
        UP_KEY = FALSE;
        RELEASED_KEY = FALSE;


        // Load the HID buffer with the key
        hid_report_in1[0] = 0x02; //keyboard hid id
        hid_report_in1[1] = 0x00;
        hid_report_in1[2] = 0x00;
        hid_report_in1[3] = 0x52;
        hid_report_in1[4] = 0x00;
        hid_report_in1[5] = 0x00;
        hid_report_in1[6] = 0x00;
        hid_report_in1[7] = 0x00;
        hid_report_in1[8] = 0x00;

    }
    else if ((DOWN_KEY == TRUE)&& (RELEASED_KEY == TRUE))
    {
        DOWN_KEY = FALSE;
        RELEASED_KEY = FALSE;


       // Load the HID buffer with the key
        hid_report_in1[0] = 0x02; //keyboard hid id
        hid_report_in1[1] = 0x00;
        hid_report_in1[2] = 0x00;
        hid_report_in1[3] = 0x51;
        hid_report_in1[4] = 0x00;
        hid_report_in1[5] = 0x00;
        hid_report_in1[6] = 0x00;
        hid_report_in1[7] = 0x00;
        hid_report_in1[8] = 0x00;

    }
    else if ((ENTER_KEY == TRUE)&& (RELEASED_KEY == TRUE))
    {
        ENTER_KEY = FALSE;
        RELEASED_KEY = FALSE;
       
        // Load the HID buffer with the key
        hid_report_in1[0] = 0x02; //keyboard hid id
        hid_report_in1[1] = 0x00;
        hid_report_in1[2] = 0x00;
        hid_report_in1[3] = 0x28;
        hid_report_in1[4] = 0x00;
        hid_report_in1[5] = 0x00;
        hid_report_in1[6] = 0x00;
        hid_report_in1[7] = 0x00;
        hid_report_in1[8] = 0x00;

    }
    // release the key -- send the "absence of key command"
    else if (RELEASED_KEY == FALSE)
    {

        RELEASED_KEY = TRUE;

        //release the key
        hid_report_in1[0] = 0x02; //keyboard hid id
        hid_report_in1[1] = 0x00;
        hid_report_in1[2] = 0x00;
        hid_report_in1[3] = 0x00;
        hid_report_in1[4] = 0x00;
        hid_report_in1[5] = 0x00;
        hid_report_in1[6] = 0x00;
        hid_report_in1[7] = 0x00;
        hid_report_in1[8] = 0x00;
        MOUSE_MOVEMENT_ALLOWED = TRUE;
 
     }
    
    return;
 }


/****************************************************************************************************************************
 *  void Emulate_Mouse(void):
 *   This function constructs the mouse HID report
 ****************************************************************************************************************************/
void Emulate_Mouse(void)
{
     // Set HID ID for mouse
     hid_report_in1[0] = 0x01;

     // Set click byte
     hid_report_in1[1] = left_click;

     // Set movement bytes
     if (MOUSE_MOVEMENT_ALLOWED == TRUE)
     {
         hid_report_in1[2] = X;
         hid_report_in1[3] = Y;
     }
     else
     {
        hid_report_in1[2] = 0;
        hid_report_in1[3] = 0;
     }

     // When click event, no movement
     if (left_click > 0)
     {
         hid_report_in1[2] = 0;
         hid_report_in1[3] = 0;
     }

     // These bytes are not associated with a mouse
      hid_report_in1[4] = 0x0;
      hid_report_in1[5]= 0x0;
      hid_report_in1[6]= 0x0;
      hid_report_in1[7]= 0x0;
      hid_report_in1[8]= 0x0;

     // Reset the click byte to no click state
     left_click  = 0x00;
}